# -*- coding: utf-8 -*-

import xarray as xr
import numpy as np


# Land-sea mask
def load_lsm(resolution, root):
    data_path = "%s/constants/constants_%s.nc" % (root, resolution)
    ds = xr.open_dataset(data_path)
    ds = ds.transpose("lat", "lon")
    dt = np.array(ds["lsm"].data, dtype=np.float32)
    return dt


# Soil type
def load_slt(resolution, root):
    data_path = "%s/constants/constants_%s.nc" % (root, resolution)
    ds = xr.open_dataset(data_path)
    ds = ds.transpose("lat", "lon")
    dt = np.array(ds["slt"].data, dtype=np.float32)
    return dt


# Orography
def load_orography(resolution, root):
    data_path = "%s/constants/constants_%s.nc" % (root, resolution)
    ds = xr.open_dataset(data_path)
    ds = ds.transpose("lat", "lon")
    dt = np.array(ds["orography"].data, dtype=np.float32)
    return dt


# Latitude
def load_lat2d(resolution, root):
    data_path = "%s/constants/constants_%s.nc" % (root, resolution)
    ds = xr.open_dataset(data_path)
    ds = ds.transpose("lat", "lon")
    dt = np.array(ds["lat2d"].data, dtype=np.float64)
    return dt


# Longitude
def load_lon2d(resolution, root):
    data_path = "%s/constants/constants_%s.nc" % (root, resolution)
    ds = xr.open_dataset(data_path)
    ds = ds.transpose("lat", "lon")
    dt = np.array(ds["lon2d"].data, dtype=np.float64)
    return dt


# Area weight
def load_area_weight(resolution, root):
    data_path = "%s/constants/constants_%s.nc" % (root, resolution)
    ds = xr.open_dataset(data_path)
    ds = ds.transpose("lat", "lon")
    lat = np.array(ds["lat2d"].data, dtype=np.float64)
    dt = np.cos(lat * np.pi / 180)
    dt = dt / dt.mean()
    return dt



# -*- coding: utf-8 -*-

import importlib
import logging
import os
import resource
from pathlib import Path

import arrow
import numpy as np
import sys
import torch as th
from torch.utils.data import DataLoader

from wxbtool.util.plotter import plot

rlimit = resource.getrlimit(resource.RLIMIT_NOFILE)
resource.setrlimit(resource.RLIMIT_NOFILE, (2048, rlimit[1]))

print("cudnn:", th.backends.cudnn.version())

np.core.arrayprint._line_width = 150
np.set_printoptions(linewidth=np.inf)

early_stopping = 5
best = np.inf
count = 0


def test_model(opt, mdl, logger=None):
    try:
        if opt.load != "":
            dump = th.load(opt.load, map_location="cpu")
            mdl.load_state_dict(dump)
    except ImportError as e:
        logger.exception(e)
        raise e

    if th.cuda.is_available():
        mdl = mdl.cuda()

    def test(mdl):
        mdl.eval()

        dataloader = DataLoader(
            mdl.dataset_test,
            batch_size=opt.batch_size,
            shuffle=True,
            num_workers=opt.n_cpu,
        )
        loss_per_epoch = 0.0
        rmse_per_epoch_t = 0.0
        inputs, targets, results = None, None, None
        for step, sample in enumerate(dataloader):
            inputs, targets = sample

            inputs = {
                v: th.as_tensor(inputs[v], dtype=th.float32) for v in mdl.setting.vars
            }
            targets = {
                v: th.as_tensor(targets[v], dtype=th.float32) for v in mdl.setting.vars
            }

            if th.cuda.is_available():
                inputs = {v: inputs[v].cuda() for v in mdl.setting.vars}
                targets = {v: targets[v].cuda() for v in mdl.setting.vars}
                mdl.constant = mdl.constant.cuda()
                mdl.weight = mdl.weight.cuda()

            with th.no_grad():
                results = mdl(*[], **inputs)
                loss = mdl.lossfun(inputs, results, targets)
                logger.info(f"Step: {step + 1:03d} | Loss: {loss.item()}")
                loss_per_epoch += loss.item() * list(results.values())[0].size()[0]

                _, tgt = mdl.get_targets(**targets)
                _, rst = mdl.get_results(**results)
                tgt = tgt.detach().cpu().numpy().reshape(-1, 1, 32, 64)
                rst = rst.detach().cpu().numpy().reshape(-1, 1, 32, 64)
                mdl.eva.weight = mdl.eva.weight.reshape(1, 1, 32, 64)
                mdl.eva.weight = mdl.eva.weight * (mdl.eva.weight > 0)
                rmse = mdl.eva.weighted_rmse(rst, tgt)
                logger.info(
                    f"Step: {step + 1:03d} | Loss: {loss.item()} | RMSE: {rmse}"
                )
                rmse_per_epoch_t += np.nan_to_num(
                    rmse * list(results.values())[0].size()[0]
                )

        rmse_total = rmse_per_epoch_t / mdl.test_size
        logger.info(f"Test Loss: {loss_per_epoch / mdl.test_size}")
        logger.info(f"Test RMSE: {rmse_total}")

        vars_in, _ = mdl.get_inputs(**inputs)
        for bas, var in enumerate(mdl.setting.vars_in):
            for ix in range(mdl.setting.input_span):
                img = vars_in[var][0, ix].detach().cpu().numpy().reshape(32, 64)
                plot(var, open("%s_inp_%d.png" % (var, ix), mode="wb"), img)

        vars_fc, _ = mdl.get_results(**results)
        vars_tg, _ = mdl.get_targets(**targets)
        for bas, var in enumerate(mdl.setting.vars_out):
            for ix in range(mdl.setting.pred_span):
                fcst = vars_fc[var][0, ix].detach().cpu().numpy().reshape(32, 64)
                tgrt = vars_tg[var][0, ix].detach().cpu().numpy().reshape(32, 64)
                plot(var, open("%s_fcs_%d.png" % (var, ix), mode="wb"), fcst)
                plot(var, open("%s_tgt_%d.png" % (var, ix), mode="wb"), tgrt)

    try:
        test(mdl)
    except Exception as e:
        print(e)
        logger.exception(e)


def main(context, opt):
    try:
        os.environ["CUDA_VISIBLE_DEVICES"] = opt.gpu
        sys.path.insert(0, os.getcwd())
        mdm = importlib.import_module(opt.module, package=None)

        name = mdm.model.name
        time_str = arrow.now().format("YYYYMMDD_HHmmss")
        model_path = Path(f"./trains/{name}-{time_str}")
        model_path.mkdir(exist_ok=True, parents=True)
        log_file = model_path / Path("train.log")

        logging.basicConfig(level=logging.INFO, filename=log_file, filemode="w")
        logger = logging.getLogger()
        logger.setLevel(logging.INFO)
        handler = logging.StreamHandler(sys.stderr)
        handler.setFormatter(
            logging.Formatter("%(asctime)s %(levelname)s %(name)s: %(message)s")
        )
        logger.addHandler(handler)
        logger.info(str(opt))
        if mdm is not None:
            if opt.data != "":
                mdm.model.load_dataset("test", "client", url=opt.data)
            else:
                mdm.model.load_dataset("test", "server")
            test_model(opt, mdm.model, logger=logger)
        print("Test Finished!")
    except ImportError as e:
        exc_info = sys.exc_info()
        print(e)
        print("failure when loading model")
        import traceback

        traceback.print_exception(*exc_info)
        del exc_info
        sys.exit(-1)



# -*- coding: utf-8 -*-

import unittest
import os
import sys
import pathlib
import unittest.mock as mock

from unittest.mock import patch


class TestTest(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    @mock.patch.dict(
        os.environ, {"WXBHOME": str(pathlib.Path(__file__).parent.absolute())}
    )
    def test_teste3d(self):
        import wxbtool.wxb as wxb

        testargs = ["wxb", "test", "-m", "models.fast_3d", "-b", "1"]
        with patch.object(sys, "argv", testargs):
            wxb.main()

    @mock.patch.dict(
        os.environ, {"WXBHOME": str(pathlib.Path(__file__).parent.absolute())}
    )
    def test_teste6d(self):
        import wxbtool.wxb as wxb

        testargs = ["wxb", "test", "-m", "models.fast_6d", "-b", "1"]
        with patch.object(sys, "argv", testargs):
            wxb.main()

    @mock.patch.dict(
        os.environ, {"WXBHOME": str(pathlib.Path(__file__).parent.absolute())}
    )
    def test_teste10d(self):
        import wxbtool.wxb as wxb

        testargs = ["wxb", "test", "-m", "models.fast_10d", "-b", "1"]
        with patch.object(sys, "argv", testargs):
            wxb.main()



# -*- coding: utf-8 -*-

import unittest
import os
import sys
import pathlib
import unittest.mock as mock

from unittest.mock import patch


class TestTrain(unittest.TestCase):

    @mock.patch.dict(
        os.environ, {"WXBHOME": str(pathlib.Path(__file__).parent.absolute())}
    )
    def test_train3d(self):
        import wxbtool.wxb as wxb

        testargs = ["wxb", "train", "-m", "models.fast_3d", "-b", "1", "-n", "1"]
        with patch.object(sys, "argv", testargs):
            wxb.main()

    @mock.patch.dict(
        os.environ, {"WXBHOME": str(pathlib.Path(__file__).parent.absolute())}
    )
    def test_train6d(self):
        import wxbtool.wxb as wxb

        testargs = ["wxb", "train", "-m", "models.fast_6d", "-b", "1", "-n", "1"]
        with patch.object(sys, "argv", testargs):
            wxb.main()

    @mock.patch.dict(
        os.environ, {"WXBHOME": str(pathlib.Path(__file__).parent.absolute())}
    )
    def test_train10d(self):
        import wxbtool.wxb as wxb

        testargs = ["wxb", "train", "-m", "models.fast_10d", "-b", "1", "-n", "1"]
        with patch.object(sys, "argv", testargs):
            wxb.main()



# -*- coding: utf-8 -*-

import unittest
import numpy as np


class TestWindowArray(unittest.TestCase):

    def test_window_access(self):
        from wxbtool.data.dataset import WindowArray

        w = WindowArray(
            np.arange(4 * 2 * 3, dtype=np.float32).reshape(4, 2, 3), shift=0, step=2
        )
        self.assertEqual((2, 2, 3), w.shape)
        self.assertEqual((2, 3), w[0].shape)
        self.assertEqual(0, np.array(w[0, 0, 0], dtype=np.float32))
        self.assertEqual(1, np.array(w[0, 0, 1], dtype=np.float32))
        self.assertEqual(2, np.array(w[0, 0, 2], dtype=np.float32))
        self.assertEqual(3, np.array(w[0, 1, 0], dtype=np.float32))
        self.assertEqual(4, np.array(w[0, 1, 1], dtype=np.float32))
        self.assertEqual(5, np.array(w[0, 1, 2], dtype=np.float32))

        w = WindowArray(
            np.arange(4 * 2 * 3, dtype=np.float32).reshape(4, 2, 3), shift=1, step=2
        )
        self.assertEqual((2, 2, 3), w.shape)
        self.assertEqual((2, 3), w[0].shape)
        self.assertEqual(6, np.array(w[0, 0, 0], dtype=np.float32))
        self.assertEqual(7, np.array(w[0, 0, 1], dtype=np.float32))
        self.assertEqual(8, np.array(w[0, 0, 2], dtype=np.float32))
        self.assertEqual(9, np.array(w[0, 1, 0], dtype=np.float32))
        self.assertEqual(10, np.array(w[0, 1, 1], dtype=np.float32))
        self.assertEqual(11, np.array(w[0, 1, 2], dtype=np.float32))

        w = WindowArray(
            np.arange(4 * 2 * 3, dtype=np.float32).reshape(4, 2, 3), shift=0, step=2
        )
        self.assertEqual((2, 2, 3), w.shape)
        self.assertEqual((2, 3), w[0].shape)
        self.assertEqual(12, np.array(w[1, 0, 0], dtype=np.float32))
        self.assertEqual(13, np.array(w[1, 0, 1], dtype=np.float32))
        self.assertEqual(14, np.array(w[1, 0, 2], dtype=np.float32))
        self.assertEqual(15, np.array(w[1, 1, 0], dtype=np.float32))
        self.assertEqual(16, np.array(w[1, 1, 1], dtype=np.float32))
        self.assertEqual(17, np.array(w[1, 1, 2], dtype=np.float32))

        w = WindowArray(
            np.arange(4 * 2 * 3, dtype=np.float32).reshape(4, 2, 3), shift=1, step=2
        )
        self.assertEqual((2, 2, 3), w.shape)
        self.assertEqual((2, 3), w[0].shape)
        self.assertEqual(18, np.array(w[1, 0, 0], dtype=np.float32))
        self.assertEqual(19, np.array(w[1, 0, 1], dtype=np.float32))
        self.assertEqual(20, np.array(w[1, 0, 2], dtype=np.float32))
        self.assertEqual(21, np.array(w[1, 1, 0], dtype=np.float32))
        self.assertEqual(22, np.array(w[1, 1, 1], dtype=np.float32))
        self.assertEqual(23, np.array(w[1, 1, 2], dtype=np.float32))



import wxbtool.config as config


class Setting:
    def __init__(self):
        self.root = (
            config.root
        )  # The root path of WeatherBench Dataset, inject from config
        self.resolution = "5.625deg"  # The spatial resolution of the model

        self.name = "test"  # The name of the model

        self.step = 4
        self.input_span = 3
        self.pred_span = 1
        self.pred_shift = 72

        self.levels = ["300", "500", "700", "850", "1000"]
        self.height = len(self.levels)

        self.vars = [
            "geopotential",
            "toa_incident_solar_radiation",
            "2m_temperature",
            "temperature",
            "total_cloud_cover",
        ]
        self.vars_in = ["z500", "z1000", "tau", "t850", "tcc", "t2m", "tisr"]
        self.vars_out = ["t850", "z500"]

        self.years_train = [
            1980,
            1981,
            1982,
            1983,
            1984,
            1985,
            1986,
            1987,
            1988,
            1989,
            1990,
            1991,
            1992,
            1993,
            1994,
            1995,
            1996,
            1997,
            1998,
            1999,
            2000,
            2001,
            2002,
            2003,
            2004,
            2005,
            2006,
            2007,
            2008,
            2009,
            2010,
            2011,
            2012,
            2013,
            2014,
        ]
        self.years_test = [2015]
        self.years_eval = [2016, 2017, 2018]



import numpy as np
import torch as th


from wxbtool.phys.constants import WaterDensity, IceDensity


def water_mass_by_density(d):
    diameter = d / 1000
    return np.pi * diameter * diameter * diameter / 6 * WaterDensity


def ice_mass_by_density(d):
    diameter = d / 1000
    return np.pi * diameter * diameter * diameter / 6 * IceDensity


def water_reflectivity_by_density(d):
    return d * d * d * d * d * d


def ice_reflectivity_by_density(d):
    return 1.5 * d * d * d * d * d * d


def marshall_palmer(n0, d):
    return n0 * th.exp(-4.1 * d)



# -*- coding: utf-8 -*-

import os
import sys
import importlib
import logging
import resource
import arrow
import numpy as np
import torch as th

from pathlib import Path
from torch.nn.utils import clip_grad_norm_
from torch.optim.lr_scheduler import ReduceLROnPlateau
from torch.utils.data import DataLoader

from wxbtool.util.plotter import plot


rlimit = resource.getrlimit(resource.RLIMIT_NOFILE)
resource.setrlimit(resource.RLIMIT_NOFILE, (2048, rlimit[1]))

print("cudnn:", th.backends.cudnn.version())

np.core.arrayprint._line_width = 150
np.set_printoptions(linewidth=np.inf)

early_stopping = 5
best = np.inf
eval_by_epoch = np.inf
count = 0


embbed = False
scheduler = None


def train_model(
    opt, mdl, lr=0.001, wd=0.0, callback=None, model_path=None, logger=None
):
    global scheduler
    optimizer = th.optim.Adam(mdl.parameters(), lr=lr, weight_decay=wd)
    scheduler = ReduceLROnPlateau(optimizer, "min")

    try:
        if not embbed and opt.load != "":
            dump = th.load(opt.load, map_location="cpu")
            mdl.load_state_dict(dump)
    except ImportError as e:
        logger.exception(e)
        raise e

    if not embbed and th.cuda.is_available():
        mdl = mdl.cuda()

    def train(epoch, mdl):
        global eval_by_epoch
        mdl.train()
        dataloader = DataLoader(
            mdl.dataset_train,
            batch_size=opt.batch_size,
            shuffle=True,
            num_workers=opt.n_cpu,
        )
        loss_per_epoch = 0.0
        for step, sample in enumerate(dataloader):
            inputs, targets = sample

            inputs = {
                v: th.as_tensor(inputs[v], dtype=th.float32) for v in mdl.setting.vars
            }
            targets = {
                v: th.as_tensor(targets[v], dtype=th.float32) for v in mdl.setting.vars
            }

            if not embbed and th.cuda.is_available():
                inputs = {v: inputs[v].cuda() for v in mdl.setting.vars}
                targets = {v: targets[v].cuda() for v in mdl.setting.vars}
                mdl.constant = mdl.constant.cuda()
                mdl.weight = mdl.weight.cuda()

            optimizer.zero_grad()
            results = mdl(*[], **inputs)
            loss = mdl.lossfun(inputs, results, targets)

            loss.backward()
            clip_grad_norm_(mdl.parameters(), mdl.clipping_threshold)
            optimizer.step()

            logger.info(
                f"Epoch: {epoch + 1:03d} | Step: {step + 1:03d} | Loss: {loss.item()}"
            )

            loss_per_epoch += loss.item() * list(results.values())[0].size()[0]

        logger.info(
            f"Epoch: {epoch + 1:03d} | Train Loss: {loss_per_epoch / mdl.train_size}"
        )

        # evaluation
        loss_eval = evaluate(epoch)
        logger.info(f"Epoch: {epoch + 1:03d} | Eval loss: {loss_eval}")
        scheduler.step(loss_eval)
        eval_by_epoch = loss_eval

        if callback is not None:
            callback(epoch, loss_eval)

        global best, count
        if loss_eval.item() >= best:
            count += 1
        else:
            count = 0
            best = loss_eval

        if count == early_stopping:
            logger.info("early stopping reached, best loss is {:5f}".format(best))

    def evaluate(epoch):
        mdl.eval()
        dataloader = DataLoader(
            mdl.dataset_eval,
            batch_size=opt.batch_size,
            shuffle=True,
            num_workers=opt.n_cpu,
        )
        loss_per_epoch = 0.0
        rmse_per_epoch_t = 0.0
        for step, sample in enumerate(dataloader):
            inputs, targets = sample

            inputs = {
                v: th.as_tensor(inputs[v], dtype=th.float32) for v in mdl.setting.vars
            }
            targets = {
                v: th.as_tensor(targets[v], dtype=th.float32) for v in mdl.setting.vars
            }

            if th.cuda.is_available():
                inputs = {v: inputs[v].cuda() for v in mdl.setting.vars}
                targets = {v: targets[v].cuda() for v in mdl.setting.vars}
                mdl.constant = mdl.constant.cuda()
                mdl.weight = mdl.weight.cuda()

            with th.no_grad():
                results = mdl(*[], **inputs)
                loss = mdl.lossfun(inputs, results, targets)
                logger.info(
                    f"Epoch: {epoch + 1:03d} | Step: {step + 1:03d} | Loss: {loss.item()}"
                )
                loss_per_epoch += loss.item() * list(results.values())[0].size()[0]

                _, tgt = mdl.get_targets(**targets)
                _, rst = mdl.get_results(**results)
                tgt = tgt.detach().cpu().numpy().reshape(-1, mdl.setting.pred_span, 32, 64)
                rst = rst.detach().cpu().numpy().reshape(-1, mdl.setting.pred_span, 32, 64)
                rmse = np.sqrt(
                    np.mean(mdl.weight.cpu().numpy() * (rst - tgt) * (rst - tgt))
                )
                logger.info(
                    f"Epoch: {epoch + 1:03d} | Step: {step + 1:03d} | Loss: {loss.item()} | RMSE: {rmse}"
                )
                rmse_per_epoch_t += np.nan_to_num(
                    rmse * list(results.values())[0].size()[0]
                )

        rmse_total = rmse_per_epoch_t / mdl.eval_size
        logger.info(
            f"Epoch: {epoch + 1:03d} | Eval Loss: {loss_per_epoch / mdl.eval_size}"
        )
        logger.info(f"Epoch: {epoch + 1:03d} | Eval RMSE: {rmse_total}")

        vars_in, _ = mdl.get_inputs(**inputs)
        for bas, var in enumerate(mdl.setting.vars_in):
            for ix in range(mdl.setting.input_span):
                img = vars_in[var][0, ix].detach().cpu().numpy().reshape(32, 64)
                plot(var, open("%s_inp_%d.png" % (var, ix), mode="wb"), img)

        vars_fc, _ = mdl.get_results(**results)
        vars_tg, _ = mdl.get_targets(**targets)
        for bas, var in enumerate(mdl.setting.vars_out):
            for ix in range(mdl.setting.pred_span):
                fcst = vars_fc[var][0, ix].detach().cpu().numpy().reshape(32, 64)
                tgrt = vars_tg[var][0, ix].detach().cpu().numpy().reshape(32, 64)
                plot(var, open("%s_fcs_%d.png" % (var, ix), mode="wb"), fcst)
                plot(var, open("%s_tgt_%d.png" % (var, ix), mode="wb"), tgrt)

        th.save(
            mdl.state_dict(),
            model_path / f"m_rmse{rmse_total:0.8f}_epoch{epoch + 1:03d}.mdl",
        )
        glb = list(model_path.glob("*.mdl"))
        if len(glb) > 6:
            for p in sorted(glb)[-3:]:
                os.unlink(p)

        th.save(
            {
                "net": mdl.state_dict(),
                "optimizer": optimizer.state_dict(),
            },
            model_path / f"z_epoch{epoch + 1:03d}.chk",
        )
        glb = list(model_path.glob("*.chk"))
        if len(glb) > 1:
            os.unlink(sorted(glb)[0])

        return rmse_total

    try:
        for epoch in range(opt.n_epochs):
            train(epoch, mdl)
    except Exception as e:
        logger.exception(e)
        raise e


def main(context, opt):
    os.environ["CUDA_VISIBLE_DEVICES"] = opt.gpu
    try:
        sys.path.insert(0, os.getcwd())
        mdm = importlib.import_module(opt.module, package=None)

        name = mdm.model.name
        time_str = arrow.now().format("YYYYMMDD_HHmmss")
        model_path = Path(f"./trains/{name}-{time_str}")
        model_path.mkdir(exist_ok=True, parents=True)
        log_file = model_path / Path("train.log")

        logging.basicConfig(level=logging.INFO, filename=log_file, filemode="w")
        logger = logging.getLogger()
        logger.setLevel(logging.INFO)
        handler = logging.StreamHandler(sys.stderr)
        handler.setFormatter(
            logging.Formatter("%(asctime)s %(levelname)s %(name)s: %(message)s")
        )
        logger.addHandler(handler)
        logger.info(str(opt))

        if mdm is not None:
            if opt.data != "":
                mdm.model.load_dataset("train", "client", url=opt.data)
            else:
                mdm.model.load_dataset("train", "server")

            train_model(
                opt,
                mdm.model,
                lr=opt.rate,
                wd=opt.weightdecay,
                model_path=model_path,
                logger=logger,
            )

        print("Training Finished!")
    except ImportError as e:
        exc_info = sys.exc_info()
        print(e)
        print("failure when loading model")
        import traceback

        traceback.print_exception(*exc_info)
        del exc_info
        sys.exit(-1)



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="wxbtool",
    version="0.1.1",
    author="Mingli Yuan",
    author_email="mingli.yuan@gmail.com",
    description="A toolkit for WeatherBench based on PyTorch",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/mountain/wxbtool",
    project_urls={
        "Documentation": "https://packaging.python.org/tutorials/distributing-packages/",
        "Source": "https://github.com/mountain/wxbtool",
        "Tracker": "https://github.com/mountain/wxbtool/issues",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    packages=setuptools.find_packages(),
    python_requires=">=3.9",
    install_requires=[
        "torch",
        "numpy",
        "netcdf4",
        "toolz",
        "dask",
        "xarray",
        "opencv-python",
        "arrow",
        "python-decouple",
        "leibniz",
        "flask",
        "msgpack",
        "msgpack-numpy",
        "requests",
        "gunicorn",
        "arghandler",
    ],
    test_suite="pytest",
    tests_require=["pytest"],
    entry_points={
        "console_scripts": [
            "wxb = wxbtool.wxb:main",
        ]
    },
)



# -*- coding: utf-8 -*-

"""
    Demo model in wxbtool package

    This model predict t850 3-days in the future
    it cost more memory and can be fitted into one P40 gpu at batch size 64
    the weighted rmse is 2.23 K
"""

import torch as th

from leibniz.nn.activation import CappingRelu
from leibniz.nn.net import resunet
from leibniz.nn.layer.hyperbolic import HyperBottleneck

from wxbtool.specs.res5_625.t850weyn import Spec, Setting3d


class ResUNetModel(Spec):
    def __init__(self, setting):
        super().__init__(setting)
        self.name = "t850d3bg-weyn"
        self.resunet = resunet(
            setting.input_span * (len(setting.vars) + 2) + self.constant_size + 2,
            1,
            spatial=(32, 64 + 2),
            layers=5,
            ratio=-1,
            vblks=[2, 2, 2, 2, 2],
            hblks=[1, 1, 1, 1, 1],
            scales=[-1, -1, -1, -1, -1],
            factors=[1, 1, 1, 1, 1],
            block=HyperBottleneck,
            relu=CappingRelu(),
            final_normalized=False,
        )

    def forward(self, **kwargs):
        batch_size = kwargs["temperature"].size()[0]
        self.update_da_status(batch_size)

        _, input = self.get_inputs(**kwargs)
        constant = self.get_augmented_constant(input)
        input = th.cat((input, constant), dim=1)
        input = th.cat((input[:, :, :, 63:64], input, input[:, :, :, 0:1]), dim=3)

        output = self.resunet(input)

        output = output[:, :, :, 1:65]
        return {"t850": output}


setting = Setting3d()
model = ResUNetModel(setting)
